/**
 *
 *  Base64 encode / decode
 *  http://www.webtoolkit.info/javascript_base64.html
 *
 **/
const Base64 = {
  // private property
  _keyStr: "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",
  // public method for encoding
  encode: function (input) {
    let output = "";
    let chr1, chr2, chr3, enc1, enc2, enc3, enc4;
    let i = 0;
    input = Base64._utf8_encode(input);

    while (i < input.length) {
      chr1 = input.charCodeAt(i++);
      chr2 = input.charCodeAt(i++);
      chr3 = input.charCodeAt(i++);
      enc1 = chr1 >> 2;
      enc2 = (chr1 & 3) << 4 | chr2 >> 4;
      enc3 = (chr2 & 15) << 2 | chr3 >> 6;
      enc4 = chr3 & 63;

      if (isNaN(chr2)) {
        enc3 = enc4 = 64;
      } else if (isNaN(chr3)) {
        enc4 = 64;
      }

      output = output + this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) + this._keyStr.charAt(enc3) + this._keyStr.charAt(enc4);
    } // Whend


    return output;
  },
  // public method for decoding
  decode: function (input) {
    let output = "";
    let chr1, chr2, chr3;
    let enc1, enc2, enc3, enc4;
    let i = 0;
    input = input.replace(/[^A-Za-z0-9+/=]/g, "");

    while (i < input.length) {
      enc1 = this._keyStr.indexOf(input.charAt(i++));
      enc2 = this._keyStr.indexOf(input.charAt(i++));
      enc3 = this._keyStr.indexOf(input.charAt(i++));
      enc4 = this._keyStr.indexOf(input.charAt(i++));
      chr1 = enc1 << 2 | enc2 >> 4;
      chr2 = (enc2 & 15) << 4 | enc3 >> 2;
      chr3 = (enc3 & 3) << 6 | enc4;
      output = output + String.fromCharCode(chr1);

      if (enc3 != 64) {
        output = output + String.fromCharCode(chr2);
      }

      if (enc4 != 64) {
        output = output + String.fromCharCode(chr3);
      }
    } // Whend


    output = Base64._utf8_decode(output);
    return output;
  },
  // private method for UTF-8 encoding
  _utf8_encode: function (string) {
    let utftext = "";
    string = string.replace(/\r\n/g, "\n");

    for (let n = 0; n < string.length; n++) {
      const c = string.charCodeAt(n);

      if (c < 128) {
        utftext += String.fromCharCode(c);
      } else if (c > 127 && c < 2048) {
        utftext += String.fromCharCode(c >> 6 | 192);
        utftext += String.fromCharCode(c & 63 | 128);
      } else {
        utftext += String.fromCharCode(c >> 12 | 224);
        utftext += String.fromCharCode(c >> 6 & 63 | 128);
        utftext += String.fromCharCode(c & 63 | 128);
      }
    } // Next n


    return utftext;
  },
  // private method for UTF-8 decoding
  _utf8_decode: function (utftext) {
    let string = "";
    let i = 0;
    let c, c2, c3; // eslint-disable-next-line @typescript-eslint/no-unused-vars

    c = c2 = 0;

    while (i < utftext.length) {
      c = utftext.charCodeAt(i);

      if (c < 128) {
        string += String.fromCharCode(c);
        i++;
      } else if (c > 191 && c < 224) {
        c2 = utftext.charCodeAt(i + 1);
        string += String.fromCharCode((c & 31) << 6 | c2 & 63);
        i += 2;
      } else {
        c2 = utftext.charCodeAt(i + 1);
        c3 = utftext.charCodeAt(i + 2);
        string += String.fromCharCode((c & 15) << 12 | (c2 & 63) << 6 | c3 & 63);
        i += 3;
      }
    } // Whend


    return string;
  } // End Function _utf8_decode

};

const PLUGIN_NAME = "cp-visual-regression-diff";
const LINK_PREFIX = `#${PLUGIN_NAME}-`;
const OVERLAY_CLASS = `${PLUGIN_NAME}-overlay`;
var FILE_SUFFIX;

(function (FILE_SUFFIX) {
  FILE_SUFFIX["diff"] = ".diff";
  FILE_SUFFIX["actual"] = ".actual";
})(FILE_SUFFIX || (FILE_SUFFIX = {}));

const TASK = {
  getScreenshotPath: `${PLUGIN_NAME}-getScreenshotPath`,
  compareImages: `${PLUGIN_NAME}-compareImages`,
  approveImage: `${PLUGIN_NAME}-approveImage`,
  doesFileExist: `${PLUGIN_NAME}-doesFileExist`
};

const nameCacheCounter = {};
Cypress.Commands.add("matchImage", {
  prevSubject: "optional"
}, (subject, options = {}) => {
  const $el = subject;
  let title = options.title || Cypress.currentTest.titlePath.join(" ");
  if (typeof nameCacheCounter[title] === "undefined") nameCacheCounter[title] = -1;
  title += ` #${++nameCacheCounter[title]}`;
  const scaleFactor = Cypress.env("pluginVisualRegressionForceDeviceScaleFactor") === false ? 1 : 1 / window.devicePixelRatio;
  const updateImages = options.updateImages || Cypress.env("pluginVisualRegressionUpdateImages") || false;
  const imagesDir = options.imagesDir || Cypress.env("pluginVisualRegressionImagesDir") || "__image_snapshots__";
  const maxDiffThreshold = options.maxDiffThreshold || Cypress.env("pluginVisualRegressionMaxDiffThreshold") || 0.01;
  const diffConfig = options.diffConfig || Cypress.env("pluginVisualRegressionDiffConfig") || {};
  const screenshotConfig = options.screenshotConfig || Cypress.env("pluginVisualRegressionScreenshotConfig") || {};
  return cy.then(() => cy.task(TASK.getScreenshotPath, {
    title,
    imagesDir,
    specPath: Cypress.spec.relative
  }, {
    log: false
  })).then(screenshotPath => {
    let imgPath;
    return ($el ? cy.wrap($el) : cy).screenshot(screenshotPath, { ...screenshotConfig,

      onAfterScreenshot(el, props) {
        imgPath = props.path;
        screenshotConfig.onAfterScreenshot == null ? void 0 : screenshotConfig.onAfterScreenshot(el, props);
      },

      log: false
    }).then(() => imgPath);
  }).then(imgPath => cy.task(TASK.compareImages, {
    scaleFactor,
    imgNew: imgPath,
    imgOld: imgPath.replace(FILE_SUFFIX.actual, ""),
    updateImages,
    maxDiffThreshold,
    diffConfig
  }, {
    log: false
  }).then(res => ({
    res: res,
    imgPath
  }))).then(({
    res,
    imgPath
  }) => {
    const log = Cypress.log({
      name: "log",
      displayName: "Match image",
      $el
    });

    if (!res) {
      log.set("message", "Unexpected error!");
      const err = new Error("Unexpected error!"); // only way to throw & log the message properly in Cypress
      // https://github.com/cypress-io/cypress/blob/5f94cad3cb4126e0567290b957050c33e3a78e3c/packages/driver/src/cypress/error_utils.ts#L214-L216

      err.onFail = err => log.error(err);

      throw err;
    }

    if (res.error) {
      log.set("message", `${res.message}\n[See comparison](${LINK_PREFIX}${Base64.encode(encodeURIComponent(JSON.stringify({
        title,
        imgPath
      })))})`);
      log.set("consoleProps", () => res);
      const err = new Error(res.message); // only way to throw & log the message properly in Cypress
      // https://github.com/cypress-io/cypress/blob/5f94cad3cb4126e0567290b957050c33e3a78e3c/packages/driver/src/cypress/error_utils.ts#L214-L216

      err.onFail = err => log.error(err);

      throw err;
    } else {
      log.set("message", res.message);
    }
  });
});

function queueClear() {
  cy.queue.clear();
  cy.state("index", 0);
}

function queueRun() {
  // needed to run a task outside of the test processing flow
  cy.queue.run();
}

function generateOverlayTemplate(title, imgNewBase64, imgOldBase64, imgDiffBase64, wasImageNotUpdatedYet) {
  return `<div class="${OVERLAY_CLASS} runner" style="position:fixed;z-index:10;top:0;bottom:0;left:0;right:0;display:flex;flex-flow:column">
  <header style="position:static">
  <nav style="display:flex;width:100%;align-items:center;justify-content:space-between;padding:10px 15px;">
    <h2>${title} - screenshot diff</h2>
    <form>
      ${wasImageNotUpdatedYet ? `<button type="submit"><i class="fa fa-check"></i> Update screenshot</button>` : "Image was already updated, rerun test to see new comparison"}
      <button type="button" data-type="close"><i class="fa fa-times"></i> Close</button>
    <form>
  </nav>
  </header>
  <div style="padding:15px;overflow:auto">
    <div style="display:flex;justify-content:space-evenly;align-items:flex-start;gap:15px">
      <div
        style="position:relative;background:#fff;border:solid 15px #fff"
        onmouseover="this.querySelector('div').style.opacity=0,this.querySelector('img').style.opacity=1"
        onmouseleave="this.querySelector('div').style.opacity=1,this.querySelector('img').style.opacity=0"
      >
        <h3>New screenshot (hover mouse away too see the old one):</h3>
        <img style="min-width:300px;width:100%;opacity:0" src="data:image/png;base64,${imgNewBase64}" />
        <div style="position:absolute;top:0;left:0;background:#fff">
          <h3>Old screenshot (hover over to see the new one):</h3>
          <img style="min-width:300px;width:100%" src="data:image/png;base64,${imgOldBase64}" />
        </div>
      </div>
      <div style="background:#fff;border:solid 15px #fff">
        <h3>Diff between new and old screenshot</h3>
        <img style="min-width:300px;width:100%" src="data:image/png;base64,${imgDiffBase64}" />
        </div>
      </div>
    </div>
  </div>`;
}

function cachedReadFile(imageCache, path, encoding) {
  if (imageCache[path]) return Cypress.Promise.resolve(imageCache[path]);
  return cy.readFile(path, encoding, {
    log: false
  }).then(file => imageCache[path] = file);
}

before(() => {
  if (!top) return null;
  Cypress.$(`.${OVERLAY_CLASS}`, top.document.body).remove();
});
after(() => {
  const imageCache = {};
  if (!top) return null;
  Cypress.$(top.document.body).on("click", `a[href^="${LINK_PREFIX}"]`, function (e) {
    e.preventDefault();
    if (!top) return false;
    const {
      title,
      imgPath
    } = JSON.parse(decodeURIComponent(Base64.decode(e.currentTarget.getAttribute("href").substring(LINK_PREFIX.length))));
    queueClear();
    cachedReadFile(imageCache, imgPath, "base64").then(imgNew => cachedReadFile(imageCache, imgPath.replace(FILE_SUFFIX.actual, ""), "base64").then(imgOld => cachedReadFile(imageCache, imgPath.replace(FILE_SUFFIX.actual, FILE_SUFFIX.diff), "base64").then(imgDiff => cy.task(TASK.doesFileExist, {
      path: imgPath
    }, {
      log: false
    }).then(wasImageNotUpdatedYet => [imgNew, imgOld, imgDiff, wasImageNotUpdatedYet])))).then(([imgNewBase64, imgOldBase64, imgDiffBase64, wasImageNotUpdatedYet]) => {
      if (!top) return false;
      queueClear();
      Cypress.$(generateOverlayTemplate(title, imgNewBase64, imgOldBase64, imgDiffBase64, wasImageNotUpdatedYet)).appendTo(top.document.body);
      const wrapper = Cypress.$(`.${OVERLAY_CLASS}`, top.document.body);
      wrapper.on("click", 'button[data-type="close"]', function () {
        wrapper.remove();
      });
      wrapper.on("submit", "form", function (e) {
        e.preventDefault();
        cy.task(TASK.approveImage, {
          img: imgPath
        }).then(() => wrapper.remove());
        queueRun();
      });
    });
    queueRun();
    return false;
  });
});
//# sourceMappingURL=support.esm.mjs.map
